/*************************************************************************************\


 #     #   #####   #     #   #####    #####   ######      #  #######  #####   ######
 #     #  #     #  ##    #  #     #  #     #  #     #    #      #    #     #  #     #
 #  #  #  #     #  # #   #  #              #  #     #   #       #          #  #     #
 #  #  #  #######  #  #  #  # #####   #####   #     #  #   #    #     #####   #     #
 #  #  #  #     #  #   # #  #     #  #        #     #  #######  #          #  #     #
 #  #  #  #     #  #    ##  #     #  #        #     #      #    #    #     #  #     #      
  ## ##   #     #  #     #   #####   #######  ######       #    #     #####   ######


                                  Preprocessor of
                 2D Wang Tiles with Elliptical Inclusions for T3D
                           Copyright: Daniel Rypl 2019

\*************************************************************************************/

/* IMPORTANT: this file is included in wang2d4t3d.c and uses static global variables defined there !!! */

static void
process_geometry(void)
{
	point_rec point, point1, point2, point3, point4, point5, point6, center;
	point_rec *pnt = NULL, *pnt1 = NULL, *pnt2 = NULL, *pnt3 = NULL, *pnt4 = NULL;
	vector_rec vec, vec_max, vec_min;
	is_point_rec *is_point = NULL, *is_point1 = NULL, *is_point2 = NULL, *is_point3 = NULL, *is_point4 = NULL;
	double weight, coord, size;
	long bg_vertex_id, bg_curve_id, bg_curve_id_minus, bg_curve_id_plus;
	long ver_id, ver_id1, ver_id2, ver_id3, ver_id4, ver_id5, ver_id6, ver_id7, ver_id8, ver_id9, ver_id10;
	long cur_id, cur_id1, cur_id2, cor_id_aux;
	logic output_x_edge_vertex, output_y_edge_vertex;
	logic output_x_edge_curve, output_y_edge_curve;
	logic halving_x, halving_y, cross_halving;
	int sign_x, sign_y;

	x_edge_is = inclusion -> edge_is[X];
	y_edge_is = inclusion -> edge_is[Y];

	output_x_edge_vertex = NO;
	output_y_edge_vertex = NO;
	output_x_edge_curve = NO;
	output_y_edge_curve = NO;

	if(inclusion -> mat != 0){
		if(y_count == 2){            /* related to y-edge = x-side */

#ifdef USE_SIDE
			sprintf(side_spec_y, "side %d", 2 * X + x_ind + 1);
#endif

			if(tile.code[2 * X + x_ind] != 0){
				code = &code_array[tile.code[2 * X + x_ind] - 1];
				
/* CAUTION: when processing tile using a particular code for the first time on both opposite sides, special care must be taken;
	 normally, intersection of inclusion with the side of that tile, which uses the code of that side for the first time, is mirrored
	 by intersections of all other instances of that inclusion with sides having the same code;
	 but this is not possible if processed tile uses particular code for the first time on both opposite sides and if the primary instance 
	 of the inclusion is intersecting positive side;
	 the reason is that in this particular case code structure stores as side of the first occurrence the negative side, which is consequence 
	 of ordering in which tile sides are processed (negative side always preceeds opposite positive side) when storing the first use of the code;
	 thus when processing the primary instance, the intersection with negative side is not available (the patch does not exist yet) 
	 because the secondary instance, which intersects in this case the negative side (which is the stored first time side), was not processed yet, 
	 which is the consequence of the fact that secondary instances are processed after the primary instance;
	 therefore the mirroring of the intersection of the primary instance with positive tile side by the intersection of the secondary instance 
	 with negative side must be used in this particular case !!!
	 in other words, when processing the primary instance, no mirroring is applied;
	 when processing the secondary instance, mirror of the already established intersesection of the primary instance with the positive side is applied;
	 this should have no impact on ordering of nodes on related sides !!!

   XN           XP    code(XN) = code(XP)
   |            |

	 o------------o     p - primary instance
   |            |     s - secondary instance
   s            p
   s   <-----   p     <--- direction of mirroring 
   s            p
   |            |
   |            |
   o------------o */

				if(code -> tile_id == tile_id){
					if(code -> id == tile.code[2 * X + 1 - x_ind]){
						if(x_ind != ind_x){                /* <==> per_x == 1 <==> secondary instance of inclusion */
						  //cur_id = ((y_edge_curve[x_ind] + 1) * max_tiles + tile_id) * 10 + 2 * X + 1 - x_ind + 1;      /* keep compatible with get_x_side_curve_id macro */
							cur_id = ((y_edge_curves[x_ind] + 1) * max_tiles + tile_id) * 10 + 2 * X + ind_x + 1;         /* keep compatible with get_x_side_curve_id macro */
#ifdef USE_SIMPLE
							sprintf(mirror_spec_y, "mirror %ld simple", cur_id);
#else
							sprintf(mirror_spec_y, "mirror %ld", cur_id);
#endif
						}
					}
				}
				else{
					cur_id = ((y_edge_curves[x_ind] + 1) * max_tiles + code -> tile_id) * 10 + code -> side + 1;    /* keep compatible with get_x_side_curve_id macro */
#ifdef USE_SIMPLE
					sprintf(mirror_spec_y, "mirror %ld simple", cur_id);
#else
					sprintf(mirror_spec_y, "mirror %ld", cur_id);
#endif
				}

#if 0
				if(interface == YES){

/* outer curves corresponding to inclusion intersection curves are "holes" not subjected to output, thus not dicretized ==> no mirroring */

				}
#endif

			}
		}

		if(x_count == 2){

#ifdef USE_SIDE
			sprintf(side_spec_x, "side %d", 2 * Y + y_ind + 1);
#endif

			if(tile.code[2 * Y + y_ind] != 0){
				code = &code_array[tile.code[2 * Y + y_ind] - 1];
				
/* CAUTION: when processing tile using a particular code for the first time on both opposite sides, special care must be taken;
	 (see comment above) */

				if(code -> tile_id == tile_id){
					if(code -> id == tile.code[2 * Y + 1 - y_ind]){
						if(y_ind != ind_y){                /* <==> per_y == 1 <==> secondary instance of inclusion */
						  //cur_id = ((x_edge_curve[y_ind] + 1) * max_tiles + tile_id) * 10 + 2 * Y + 1 - y_ind + 1;      /* keep compatible with get_y_side_curve_id macro */
							cur_id = ((x_edge_curves[y_ind] + 1) * max_tiles + tile_id) * 10 + 2 * Y + ind_y + 1;         /* keep compatible with get_y_side_curve_id macro */
#ifdef USE_SIMPLE
							sprintf(mirror_spec_x, "mirror %ld simple", cur_id);
#else
							sprintf(mirror_spec_x, "mirror %ld", cur_id);
#endif
						}
					}
				}
				else{
					cur_id = ((x_edge_curves[y_ind] + 1) * max_tiles + code -> tile_id) * 10 + code -> side + 1;    /* keep compatible with get_y_side_curve_id macro */
#ifdef USE_SIMPLE
					sprintf(mirror_spec_x, "mirror %ld simple", cur_id);
#else
					sprintf(mirror_spec_x, "mirror %ld", cur_id);
#endif
				}

#if 0
				if(interface == YES){

/* outer curves corresponding to inclusion intersection curves are "holes" not subjected to output, thus not dicretized ==> no mirroring */

				}
#endif

			}
		}
	}

	add_vec(center, ellipse -> center, shift_vec);            /* shift ellipse center */
					
/* IMPORTANT: do not return after processing particular branch of code;
	 there are checks at the end of the routine !!! */

	if(x_count == 0 && y_count == 0){        /* not crossing edges */
		if((face_is -> side[X] & ref_position[x_ind]) != 0 && (face_is -> side[Y] & ref_position[y_ind]) != 0){
			copy_mul_vec(vec_max, ellipse -> major, ellipse -> max);
			copy_mul_vec(vec_min, ellipse -> minor, 2.0 * ellipse -> min);

/* I do rely on fact that center is already shifted */

			add_vec(point1, center, vec_max);
			sub_vec(point2, center, vec_max);
			add_vec(point3, point1, vec_min);
			add_vec(point4, point2, vec_min);
			sub_vec(point5, point1, vec_min);
			sub_vec(point6, point2, vec_min);

			if(interface == YES){  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
				write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1, ver_id2, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
				write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id1, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
				
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
			}

			write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
			write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
			write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1, ver_id2, CURVE_RATE, inclusion -> bmsz, tile_id);
			write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
			write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
			write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id1, CURVE_RATE, inclusion -> bmsz, tile_id);
			write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
			write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

			if(inclusion -> mat == 0 || interface == YES)
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, cur_id1, cur_id2, tile_id);
			else
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

			tile_patch_array[tile_patches++] = patch_id;

			if(interface == YES){
				write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 2 * 10, tile_id, inclusion -> ifmat);
				write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 2 * 10, tile_id, inclusion -> ifmat);
			}
		}
	}
	else if(x_count == 1 && y_count == 0){                        /* touching x-edge */
		if((x_edge_is -> side & ref_position[x_ind]) != 0){
			if((face_is -> side[Y] & ref_position[y_ind]) != 0){      /* touching x-edge from inside */
				pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);            /* y- */
				pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);            /* y+ */
				
				size = fabs(pnt1 -> y - pnt2 -> y);
				size = ellipse -> max * ellipse -> min / size;          /* constant area defined by associated diameters */
				
				add_vec(point1, *pnt1, shift_vec);
				add_vec(point2, *pnt2, shift_vec);
				copy_vec(point3, point1);
				copy_vec(point4, point2);
				copy_vec(point5, point1);
				copy_vec(point6, point2);
				
				is_point = &(x_edge_is -> point[0]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */
					
/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
					
				if(ver_id == 0){
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
					
					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point -> id = (y_ind == 0) ? ver_id1 : ver_id2;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point -> data);
					output_x_edge_vertex = YES;
					x_edge_curves_2[y_ind]++;
				}
				else{
					if(y_ind == 0){
						ver_id1 = ver_id;
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
					}
					else{
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
						ver_id2 = ver_id;
					}
				}
				
				point3.x += 4.0 * size;
				point4.x += 4.0 * size;
				point5.x -= 4.0 * size;
				point6.x -= 4.0 * size;
				
				if(interface == YES){  /* inner */
					write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1 - 1, ver_id2 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
					write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2 - 1, ver_id1 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
				}

				write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1, ver_id2, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
				write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id1, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

				if(inclusion -> mat == 0 || interface == YES)
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, cur_id1, cur_id2, tile_id);
				else
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

				tile_patch_array[tile_patches++] = patch_id;

				if(interface == YES){
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 2 * 10, tile_id, inclusion -> ifmat);
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 2 * 10, tile_id, inclusion -> ifmat);
				}
			}
			else{                                                            /* touching x-edge from outside */
				is_point = &(x_edge_is -> point[0]);
				ver_id = is_point -> id;
					
				if(ver_id == 0){
					pnt = &(face_is -> extreme_pnt[2 * Y + 1 - y_ind]);          /* y- or y+ on x-edge */
					add_vec(point, *pnt, shift_vec);
							
/* note: interface is irrelevant */

					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);

					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point -> id = ver_id;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point -> data);
					output_x_edge_vertex = YES;
					x_edge_curves_2[y_ind]++;
				}
			}
		}
	}
	else if(x_count == 0 && y_count == 1){                        /* touching y-edge */
		if((y_edge_is -> side & ref_position[y_ind]) != 0){
			if((face_is -> side[X] & ref_position[x_ind]) != 0){      /* touching y-edge from inside */
				pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);            /* x- */
				pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);            /* x+ */
						
				size = fabs(pnt1 -> x - pnt2 -> x);
				size = ellipse -> max * ellipse -> min / size;          /* constant area defined by associated diameters */
					
				add_vec(point1, *pnt1, shift_vec);
				add_vec(point2, *pnt2, shift_vec);
				copy_vec(point3, point1);
				copy_vec(point4, point2);
				copy_vec(point5, point1);
				copy_vec(point6, point2);

				is_point = &(y_edge_is -> point[0]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */
					
/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

				if(ver_id == 0){
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						
					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point -> id = (x_ind == 0) ? ver_id1 : ver_id2;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point -> data);
					output_y_edge_vertex = YES;
					y_edge_curves_2[x_ind]++;
				}
				else{
					if(x_ind == 0){
						ver_id1 = ver_id;
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
					}
					else{
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
						ver_id2 = ver_id;
					}
				}
					
				point3.y -= 4.0 * size;
				point4.y -= 4.0 * size;
				point5.y += 4.0 * size;
				point6.y += 4.0 * size;

				if(interface == YES){  /* inner */
					write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1 - 1, ver_id2 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
					write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2 - 1, ver_id1 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
					write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
						
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
				}
					
				write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1, ver_id2, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
				write_record("curve %ld vertex %ld %ld order 4 rate %g size %g density 2 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id1, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
				write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
					
				if(inclusion -> mat == 0 || interface == YES)
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, cur_id1, cur_id2, tile_id);
				else
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

				tile_patch_array[tile_patches++] = patch_id;

				if(interface == YES){
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 2 * 10, tile_id, inclusion -> ifmat);
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 2 * 10, tile_id, inclusion -> ifmat);
				}
			}
			else{                                                            /* touching y-edge from outside */
				is_point = &(y_edge_is -> point[0]);
				ver_id = is_point -> id;
						
				if(ver_id == 0){
					pnt = &(face_is -> extreme_pnt[2 * X + 1 - x_ind]);          /* x- or x+ on y-edge */
					add_vec(point, *pnt, shift_vec);
						
/* note: interface is irrelevant */

					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);
							
					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point -> id = ver_id;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point -> data);
					output_y_edge_vertex = YES;
					y_edge_curves_2[x_ind]++;
				}
			}
		}
	}
	else if(x_count == 1 && y_count == 1){                   /* touching both x-edge and y-edge */
		if((x_edge_is -> side & ref_position[x_ind]) != 0 && (y_edge_is -> side & ref_position[y_ind]) != 0){
			pnt1 = &(face_is -> extreme_pnt[2 * Y + y_ind]);     /* y- or y+ on x-edge */
			pnt2 = &(face_is -> extreme_pnt[2 * X + x_ind]);     /* x- or x+ on y-edge */
					
			add_vec(point1, *pnt1, shift_vec);
			add_vec(point2, *pnt2, shift_vec);
				
			point.x = point2.x;
			point.y = point1.y;
				
			weight = face_is -> weight;
				
			is_point1 = &(x_edge_is -> point[0]);
			ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
				
			is_point2 = &(y_edge_is -> point[0]);
			ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
			if(ver_id1 == 0){
				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					
				x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
				x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
				output_x_edge_vertex = YES;
				x_edge_curves_2[y_ind]++;
			}
			if(ver_id2 == 0){
				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
					
				y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
				y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
				output_y_edge_vertex = YES;
				y_edge_curves_2[x_ind]++;
			}
				
			if(interface == YES){  /* inner */
				if(x_ind == y_ind ){
					weight = sqrt(1.0 - weight * weight);
						
					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id2 - 1, ver_id1 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id1 - 1, ver_id2 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
				}
				else{
					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1 - 1, ver_id2 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2 - 1, ver_id1 - 1, tile_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
				}
					
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
			}

			if(x_ind == y_ind ){
				if(interface == NO)weight = sqrt(1.0 - weight * weight);
						
				write_record("curve %ld vertex %ld %ld order 3 rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id2, ver_id1, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
				write_record("curve %ld vertex %ld %ld order 3 rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id1, ver_id2, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
			}
			else{
				write_record("curve %ld vertex %ld %ld order 3 rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id1, ver_id2, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
				write_record("curve %ld vertex %ld %ld order 3 rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id1, CURVE_RATE, inclusion -> bmsz, tile_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
			}
				
			if(inclusion -> mat == 0 || interface == YES)
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, cur_id1, cur_id2, tile_id);
			else
				write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

			tile_patch_array[tile_patches++] = patch_id;

			if(interface == YES){
				write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 2 * 10, tile_id, inclusion -> ifmat);
				write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 2 * 10, tile_id, inclusion -> ifmat);
			}
		}
		else{
			if((x_edge_is -> side & ref_position[x_ind]) != 0){
				is_point1 = &(x_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;
					
				if(ver_id1 == 0){
					pnt1 = &(face_is -> extreme_pnt[2 * Y + 1 - y_ind]);          /* y- or y+ on x-edge */
					add_vec(point1, *pnt1, shift_vec);
						
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
						
					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
					output_x_edge_vertex = YES;
					x_edge_curves_2[y_ind]++;
				}
			}
			if((y_edge_is -> side & ref_position[y_ind]) != 0){
				is_point2 = &(y_edge_is -> point[0]);
				ver_id2 = is_point2 -> id;
					
				if(ver_id2 == 0){
					pnt2 = &(face_is -> extreme_pnt[2 * X + 1 - x_ind]);          /* x- or x+ on y-edge */
					add_vec(point2, *pnt2, shift_vec);
						
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						
					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
					output_y_edge_vertex = YES;
					y_edge_curves_2[x_ind]++;
				}
			}
		}
	}
	else if(x_count == 2 && y_count == 0){                   /* crossing x-edge */
		if((x_edge_is -> side & ref_position[x_ind]) != 0){
			pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);         /* x- */
			pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);         /* x+ */

			is_point1 = &(x_edge_is -> point[0]);
			ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */

			is_point2 = &(x_edge_is -> point[1]);
			ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */

			coord = tile.corner[ind_y].y;  /* caution: ind_y is correct !!! */

			point1.y = point2.y = par_corner[y_ind].y;

			point1.x = is_point1 -> data.coord + shift_x; 
			point2.x = is_point2 -> data.coord + shift_x; 
					
			if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
				if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
					
				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
				
				x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
				x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
				x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point2 -> id = ver_id2;
				x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point2 -> data);
				output_x_edge_vertex = YES;
				x_edge_curves_2[y_ind]++;
			}
			
			x_curve_id = get_y_side_curve_id(y_ind);
			if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

				write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, ver_id2 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
				x_curve_id += 4;
				write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
			}
			else{
				write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_x, side_spec_x);
			}

			x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
			output_x_edge_curve = YES;

			sign_x = (y_ind == 1) ? -1 : 1;

			halving_x = YES;
			if(face_is -> edge_half[X] == NO){
				if(y_ind == 0){
					if(pnt1 -> y <= coord || pnt2 -> y <= coord){    /* CAUTION: floating point number based decision */
						halving_x = NO;

						weight = face_is -> weight;
						if(pnt1 -> y <= pnt2 -> y){
							pnt3 = pnt1;
							weight = sqrt(1.0 - weight * weight);
						}
						else
							pnt3 = pnt2;
						pnt4 = &(face_is -> extreme_pnt[2 * Y + 0]);
					}
				}
				else{
					if(pnt1 -> y >= coord || pnt2 -> y >= coord){    /* CAUTION: floating point number based decision */
						halving_x = NO;

						weight = face_is -> weight;
						if(pnt1 -> y <= pnt2 -> y){
							pnt3 = pnt2;
							weight = sqrt(1.0 - weight * weight);
						}
						else
							pnt3 = pnt1;
						pnt4 = &(face_is -> extreme_pnt[2 * Y + 1]);
					}
				}
			}

			if(halving_x == NO){
				add_vec(point3, *pnt3, shift_vec);
				add_vec(point4, *pnt4, shift_vec);

				point.x = point3.x;
				point.y = point4.y;
					
				write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point3.x, point3.y, 0.0);
				write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point4.x, point4.y, 0.0);
				write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, vertex_id - 1, vertex_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
											
				if(interface == YES){  /* inner */
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);

					if(y_ind == 0)
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, tile_id);
					else
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, tile_id);
						
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
				}

				write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
				write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

				if(y_ind == 0)
					write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
				else
					write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					
				if(inclusion -> mat == 0 || interface == YES)
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
				else
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

				if(interface == YES){
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
				}
			}
			else{
#ifdef DEBUG_PRINT
				fprintf(stderr, "halving with respect to x\n\n");
#endif
				pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
				pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
									
				size = fabs(pnt1 -> y - pnt2 -> y);
				size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
				if((y_ind == 0 && center.y > par_corner[0].y) || (y_ind == 1 && center.y < par_corner[1].y)){          /* CAUTION */
#else
					if(center.y * sign[y_ind] > par_corner[y_ind].y * sign[y_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to x-edge */

						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
						copy_vec(point3, point1);
						copy_vec(point4, point2);
						copy_vec(point5, point1);
						copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

						if(y_ind == 0){
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						}
						else{
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
						}
						
						point3.x += 4.0 * size;
						point4.x += 4.0 * size;
						point5.x -= 4.0 * size;
						point6.x -= 4.0 * size;
				
/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

						if(interface == YES){  /* inner */
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4 - 1, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id6, bg_curve_id, tile_id);

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
								
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id5, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3 - 1, bg_curve_id, tile_id);
							
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
						}
						
						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
						
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
						
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						
						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
						
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
						
						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{ 

/* halving parallel to x-edge */

						point1.y = point2.y = center.y;
							
						point1.x = center.x - 2.0 * size;
						point2.x = center.x + 2.0 * size;
							
						if(y_ind == 0){
							sub_vec(vec, *pnt2, *pnt1);
						}
						else{
							sub_vec(vec, *pnt1, *pnt2);
						}
							
						add_vec(point3, point1, vec);
						add_vec(point4, point2, vec);
							
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							
						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
							
						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
#if 0 /* only to fix indentation */
					}
#else
				}
#endif
			}
			tile_patch_array[tile_patches++] = patch_id;
		}
	}
	else if(x_count == 0 && y_count == 2){                      /* crossing y-edge */
		if((y_edge_is -> side & ref_position[y_ind]) != 0){
			pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);            /* y- */
			pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);            /* y+ */

			is_point1 = &(y_edge_is -> point[0]);
			ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */

			is_point2 = &(y_edge_is -> point[1]);
			ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */

			coord = tile.corner[ind_x].x;  /* caution: ind_x is correct !!! */

			point1.x = point2.x = par_corner[x_ind].x;

			point1.y = is_point1 -> data.coord + shift_y; 
			point2.y = is_point2 -> data.coord + shift_y; 
					
			if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
				if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
				if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
				write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);

				y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point1 -> id = ver_id1;
				y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point1 -> data);
				y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
				y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
				output_y_edge_vertex = YES;
				y_edge_curves_2[x_ind]++;
			}

			y_curve_id = get_x_side_curve_id(x_ind);
			if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

				write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1 - 1, ver_id2 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
				y_curve_id += 4;
				write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
			}
			else{
				write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_y, side_spec_y);
			}

			y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
			output_y_edge_curve = YES;

			sign_y = (x_ind == 0) ? -1 : 1;

			halving_y = YES;
			if(face_is -> edge_half[Y] == NO){
				if(x_ind == 0){
					if(pnt1 -> x <= coord || pnt2 -> x <= coord){    /* CAUTION: floating point number based decision */
						halving_y = NO;

						weight = face_is -> weight;
						if(pnt1 -> x <= pnt2 -> x){
							pnt3 = pnt1;
							weight = sqrt(1.0 - weight * weight);
						}
						else
							pnt3 = pnt2;
						pnt4 = &(face_is -> extreme_pnt[2 * X + 0]);
					}
				}
				else{
					if(pnt1 -> x >= coord || pnt2 -> x >= coord){    /* CAUTION: floating point number based decision */
						halving_y = NO;

						weight = face_is -> weight;
						if(pnt1 -> x <= pnt2 -> x){
							pnt3 = pnt2;
							weight = sqrt(1.0 - weight * weight);
						}
						else
							pnt3 = pnt1;
						pnt4 = &(face_is -> extreme_pnt[2 * X + 1]);
					}
				}
			}											
				
			if(halving_y == NO){
				add_vec(point3, *pnt3, shift_vec);
				add_vec(point4, *pnt4, shift_vec);
					
				point.x = point4.x;
				point.y = point3.y;
					
				write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point3.x, point3.y, 0.0);
				write_record("vertex %ld xyz "FMT3" virtual\n", ++vertex_id, point4.x, point4.y, 0.0);
				write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, vertex_id - 1, vertex_id);
				write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
					
				if(interface == YES){  /* inner */
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
					
					if(x_ind == 0)
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, tile_id);
					else
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, tile_id);
					
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
				}

				write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
				write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

				if(x_ind == 0)
					write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
				else
					write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

				if(inclusion -> mat == 0 || interface == YES)
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
				else
					write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

				if(interface == YES){
					write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
				}
			}
			else{
#ifdef DEBUG_PRINT
				fprintf(stderr, "halving with respect to y\n\n");
#endif
				pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
				pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
									
				size = fabs(pnt1 -> x - pnt2 -> x);
				size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
				if((x_ind == 0 && center.x > par_corner[0].x) || (x_ind == 1 && center.x < par_corner[1].x)){          /* CAUTION */
#else
					if(center.x * sign[x_ind] > par_corner[x_ind].x * sign[x_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to y-edge */

						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
						copy_vec(point3, point1);
						copy_vec(point4, point2);
						copy_vec(point5, point1);
						copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

						if(x_ind == 0){
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						}
						else{
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
						}

						point3.y -= 4.0 * size;
						point4.y -= 4.0 * size;
						point5.y += 4.0 * size;
						point6.y += 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

						if(interface == YES){  /* inner */
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);
											
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
											
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{

/* halving parallel to y-edge */ 

						point1.x = point2.x = center.x;
													
						point1.y = center.y - 2.0 * size;
						point2.y = center.y + 2.0 * size;
												
						if(x_ind == 0){
							sub_vec(vec, *pnt2, *pnt1);
						}
						else{
							sub_vec(vec, *pnt1, *pnt2);
						}
										
						add_vec(point3, point1, vec);
						add_vec(point4, point2, vec);
										
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
						write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
										
						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										
						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
#if 0 /* only to fix indentation */
					}
#else
				}
#endif
			}
			tile_patch_array[tile_patches++] = patch_id;
		}
	}
	else if(x_count == 2 && y_count == 1){   /* crossing x-edge, touching y-edge */
		if((x_edge_is -> side & ref_position[x_ind]) != 0){
			halving_x = NO;
			if(face_is -> edge_half[X] == YES){

/* halving is relevant only if processing part of ellipse not touching y-edge */

				if(y_edge_is -> side != ref_position[y_ind])halving_x = YES;
			}

			if(inclusion -> corner == OUT){
				is_point = &(y_edge_is -> point[0]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */
									
				if(ver_id == 0){
					point.x = par_corner[x_ind].x;
					point.y = is_point -> data.coord + shift_y; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

					if(interface == YES){  /* inner */
						write_record("vertex %ld xyz "FMT3, ++vertex_id, point.x, point.y, 0.0);

						if(y_edge_is -> side == ref_position[y_ind])
							write_record(" tile %ld\n", tile_id);
						else
							write_record(" virtual\n");         /* caution: virtual vertex is duplicated ==> overhead */
					}

					write_record("vertex %ld xyz "FMT3, ver_id = ++vertex_id, point.x, point.y, 0.0);

					if(y_edge_is -> side == ref_position[y_ind]){
						write_record(" tile %ld\n", tile_id);
						y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point -> id = ver_id;
						y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point -> data);
						output_y_edge_vertex = YES;
						y_edge_curves_2[x_ind]++;
					}
					else
						write_record(" virtual\n");
				}

				is_point1 = &(x_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
									
				is_point2 = &(x_edge_is -> point[1]);
				ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */

				if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
					if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
					point1.y = point2.y = par_corner[y_ind].y;

					point1.x = is_point1 -> data.coord + shift_x; 
					point2.x = is_point2 -> data.coord + shift_x; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);

					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point2 -> id = ver_id2;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point2 -> data);
					output_x_edge_vertex = YES;
					x_edge_curves_2[y_ind]++;
				}

				x_curve_id = get_y_side_curve_id(y_ind);
				if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, ver_id2 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
					x_curve_id += 4;
					write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
				}
				else{
					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_x, side_spec_x);
				}
				
				x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
				output_x_edge_curve = YES;
									
				sign_x = (y_ind == 1) ? -1 : 1;

				if(halving_x == YES){
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to x\n\n");
#endif

/* half-ellipse arc with both points "on" x-edge is processed */

					pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
										
					size = fabs(pnt1 -> y - pnt2 -> y);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((y_ind == 0 && center.y > par_corner[0].y) || (y_ind == 1 && center.y < par_corner[1].y)){          /* CAUTION */
#else
						if(center.y * sign[y_ind] > par_corner[y_ind].y * sign[y_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to x-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

							if(y_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}
											
							point3.x += 4.0 * size;
							point4.x += 4.0 * size;
							point5.x -= 4.0 * size;
							point6.x -= 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id6, bg_curve_id, tile_id);

								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3 - 1, bg_curve_id, tile_id);
												
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
												
							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to x-edge */

							point1.y = point2.y = center.y;
											
							point1.x = center.x - 2.0 * size;
							point2.x = center.x + 2.0 * size;
											
							if(y_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}

							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
											
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
												
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							if(interface == YES){  /* inner */
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					//pnt1 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ */
					add_vec(point2, *pnt2, shift_vec);
										
					point.x = par_corner[x_ind].x;
					point.y = point2.y;
										
					weight = face_is -> weight;
					if(x_ind == y_ind)weight = sqrt(1.0 - weight * weight);

					write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, 0.0);

					if(y_edge_is -> side != ref_position[y_ind]){           

/* elliptical arc with both points on x-edge is processed */

/* CAUTION: although vertex ver_id is duplicated (no matter whether virtual or not), virtual curve bg_curve_id_minus is not duplicated 
	 because ver_id is not bounding any fixed (physical) curve */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id_minus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id_minus, tile_id);

							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
											
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{

/* two elliptical arcs sharing point on y-edge are processed */

/* process elliptical arc closer to tile corner at first */

/* CAUTION: virtual curve bg_curve_id_plus must be duplicated because vertex ver_id is duplicated (no matter whether virtual or not)
	 and is bounding a fixed (physical) curve */

/* CAUTION: ids of created outer and inner cur_id differ by 2 * 10 !!! */

						if(interface == YES){  /* inner */
							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_plus = ++curve_id * 10, ver_id - 1, bg_vertex_id);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
									
							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2 - 1, bg_curve_id_plus, tile_id);
								
							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id - 1, ver_id3, bg_curve_id_plus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id3, ver_id - 1, bg_curve_id_plus, tile_id);
						}

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_plus = ++curve_id * 10, ver_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
										
						if(x_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
										
						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id, ver_id3, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id3, ver_id, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);

/* then process elliptical arc further from tile corner */
									
						if(face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
							fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
							error_message(GENERAL_ERROR, "Unimplemented situation");
#endif
#ifdef DEBUG
							if(face_is -> cross_halving == X)error_message(GENERAL_ERROR, "Unexpected situation");
#endif

/* note: patch is topologically the same no matter what halving is applied;
	 but because of its duplication in case of interface, patch is formed separately for each (implemented) halving */

							if(face_is -> cross_halving == Y){                 /* halving with respect to y-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
								pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
													
								size = fabs(pnt1 -> y - pnt2 -> y);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
													
								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

								if(y_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id3 = bg_vertex_id;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id4 = bg_vertex_id;
								}
													
								point3.x += 4.0 * size;
								point4.x += 4.0 * size;
								point5.x -= 4.0 * size;
								point6.x -= 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

								if(interface == YES){  /* inner */
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id - 1, bg_curve_id, tile_id);
													
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);

									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);

									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);

									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, cur_id - 2 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
								}

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
													
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
													
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, cur_id, tile_id);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 2 * 10, tile_id, inclusion -> ifmat);
								}
							}
#ifdef MIN_MAX_HALVING
							else if(face_is -> cross_halving == MIN){
							}
							else if(face_is -> cross_halving == MAX){
							}
#endif
						}
						else{

/* CAUTION: virtual curve bg_curve_id_minus must be duplicated because vertex ver_id is duplicated (no matter whether virtual or not)
	 and is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id - 1, bg_vertex_id);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
									
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);

								if(x_ind == y_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id - 1, bg_curve_id_minus, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id - 1, ver_id4, bg_curve_id_minus, tile_id);

								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, cur_id - 2 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id, bg_vertex_id);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id4, ver_id, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id, ver_id4, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, cur_id, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 2) * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 2 * 10, tile_id, inclusion -> ifmat);
							}
						}
					}
				}
			}
			else{   /* BND */
				is_point = &(x_edge_is -> point[1 - x_ind]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

				if(ver_id == 0){
					point.x = is_point -> data.coord + shift_x; 
					point.y = par_corner[y_ind].y;

					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point.x, point.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);

					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point -> id = ver_id;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point -> data);
					output_x_edge_vertex = YES;
				}

/* I cannot ensure that ids of outer and inner corner vertices differ by 1 because corner was created at the beginning;
	 duplication of corner is done by fixing a virtual vertex to the corner and by fixing a physical vertex to the virtual vertex;
	 this implies that I do rely on virtual separation;
	 note that this concept introduces an overhead (virtual vertex is redundant) but this overhead cannot grow
	 because only one inclusion can be passing through a particular corner in a particular tile */

				if(interface == YES){   /* inner */
					if(cor_id == 0){
						write_record("vertex %ld fixed vertex %ld virtual\n", cor_id_aux = ++vertex_id, corner_id);
						write_record("vertex %ld fixed vertex %ld tile %ld\n", cor_id = ++vertex_id, cor_id_aux, tile_id);
					}
				}

				if(x_ind == 0){
					ver_id1 = corner_id;            /* outer vertex if interface == YES */
					ver_id2 = ver_id;               /* outer vertex if interface == YES */
				}
				else{
					ver_id1 = ver_id;               /* outer vertex if interface == YES */
					ver_id2 = corner_id;            /* outer vertex if interface == YES */
				}
				
				x_curve_id = get_y_side_curve_id(y_ind);
				if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

					if(x_ind == 0)
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, cor_id, ver_id2 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
					else
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, cor_id, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
					x_curve_id += 4;
					write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
				}
				else{
					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_x, side_spec_x);
				}
				
				x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
				output_x_edge_curve = YES;

				sign_x = (y_ind == 1) ? -1 : 1;
				
				if(halving_x == YES){
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to x\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
										
					size = fabs(pnt1 -> y - pnt2 -> y);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((y_ind == 0 && center.y > par_corner[0].y) || (y_ind == 1 && center.y < par_corner[1].y)){          /* CAUTION */
#else
						if(center.y * sign[y_ind] > par_corner[y_ind].y * sign[y_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to x-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
/* caution: virtual vertices are not duplicated */

							if(y_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}
											
							point3.x += 4.0 * size;
							point4.x += 4.0 * size;
							point5.x -= 4.0 * size;
							point6.x -= 4.0 * size;
				
/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id6, bg_curve_id, tile_id);

								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3 - 1, bg_curve_id, tile_id);
												
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to x-edge */

							point1.y = point2.y = center.y;
											
							point1.x = center.x - 2.0 * size;
							point2.x = center.x + 2.0 * size;
											
							if(y_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}

							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
											
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
												
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							if(interface == YES){  /* inner */
								if(x_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								}
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					weight = face_is -> weight;
					if(x_ind == y_ind)weight = sqrt(1.0 - weight * weight);
										
					//pnt1 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ on y-edge */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ */
					add_vec(point2, *pnt2, shift_vec);

					point.x = par_corner[x_ind].x;
					point.y = point2.y;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

					write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, 0.0);

					if(interface == YES){  /* inner */
						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, cor_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(x_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							
						if(y_ind == 0){
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, cor_id, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, cor_id, ver_id3, bg_curve_id, tile_id);
						}											
						else{
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, cor_id, ver_id4, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, cor_id, bg_curve_id, tile_id);
						}
											
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
					}

					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, corner_id, bg_vertex_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

					if(x_ind == 0)
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
					else
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

					if(y_ind == 0){
						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id1, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id2, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					}											
					else{
						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id1, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id2, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					}
											
					if(inclusion -> mat == 0 || interface == YES)
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
					else
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

					if(interface == YES){
						write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 2) * 10, tile_id, inclusion -> ifmat);
					}
				}
			}
			tile_patch_array[tile_patches++] = patch_id;
		}
		else{
			if(inclusion -> corner == OUT){
				if((y_edge_is -> side & ref_position[y_ind]) != 0){
					is_point = &(y_edge_is -> point[0]);
					ver_id = is_point -> id;
										
					if(ver_id == 0){
						pnt = &(face_is -> extreme_pnt[2 * X + 1 - x_ind]);          /* x- or x+ on y-edge */
						add_vec(point, *pnt, shift_vec);

						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point.x, point.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);
											
						y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point -> id = ver_id;
						y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point -> data);
						output_y_edge_vertex = YES;
						y_edge_curves_2[x_ind]++;
					}
				}
			}
		}
	}
	else if(x_count == 1 && y_count == 2){   /* crossing y-edge, touching x-edge */
		if((y_edge_is -> side & ref_position[y_ind]) != 0){
			halving_y = NO;
			if(face_is -> edge_half[Y] == YES){

/* halving is relevant only if processing part of ellipse not touching x-edge */

				if(x_edge_is -> side != ref_position[x_ind])halving_y = YES;
			}

			if(inclusion -> corner == OUT){
				is_point = &(x_edge_is -> point[0]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */
									
				if(ver_id == 0){
					point.x = is_point -> data.coord + shift_x; 
					point.y = par_corner[y_ind].y;

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

					if(interface == YES){  /* inner */
						write_record("vertex %ld xyz "FMT3, ++vertex_id, point.x, point.y, 0.0);

						if(x_edge_is -> side == ref_position[x_ind])
							write_record(" tile %ld\n", tile_id);
						else
							write_record(" virtual\n");         /* caution: virtual vertex is duplicated ==> overhead */
					}
						
					write_record("vertex %ld xyz "FMT3, ver_id = ++vertex_id, point.x, point.y, 0.0);

					if(x_edge_is -> side == ref_position[x_ind]){
						write_record(" tile %ld\n", tile_id);
						x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point -> id = ver_id;
						x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point -> data);
						output_x_edge_vertex = YES;
						x_edge_curves_2[y_ind]++;
					}
					else
						write_record(" virtual\n");
				}

				is_point1 = &(y_edge_is -> point[0]);
				ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
									
				is_point2 = &(y_edge_is -> point[1]);
				ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */

				if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
					if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
					point1.x = point2.x = par_corner[x_ind].x;

					point1.y = is_point1 -> data.coord + shift_y; 
					point2.y = is_point2 -> data.coord + shift_y; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */		
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);		

					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point1 -> id = ver_id1;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point1 -> data);
					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
					output_y_edge_vertex = YES;
					y_edge_curves_2[x_ind]++;
				}

				y_curve_id = get_x_side_curve_id(x_ind);
				if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1 - 1, ver_id2 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
					y_curve_id += 4;
					write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
				}
				else{
					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_y, side_spec_y);
				}
				
				y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
				output_y_edge_curve = YES;

				sign_y = (x_ind == 0) ? -1 : 1;

				if(halving_y == YES){
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to y\n\n");
#endif

/* half-ellipse arc with both points "on" y-edge is processed */

					pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
					pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
							
					size = fabs(pnt1 -> x - pnt2 -> x);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((x_ind == 0 && center.x > par_corner[0].x) || (x_ind == 1 && center.x < par_corner[1].x)){          /* CAUTION */
#else
						if(center.x * sign[x_ind] > par_corner[x_ind].x * sign[x_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to y-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

							if(x_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}
											
							point3.y -= 4.0 * size;
							point4.y -= 4.0 * size;
							point5.y += 4.0 * size;
							point6.y += 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to y-edge */ 

							point1.x = point2.x = center.x;
											
							point1.y = center.y - 2.0 * size;
							point2.y = center.y + 2.0 * size;
											
							if(x_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
											
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
											
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
											
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							if(interface == YES){  /* inner */
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
											
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					//pnt1 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ on x-edge */
					pnt2 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					add_vec(point2, *pnt2, shift_vec);
										
					point.x = point2.x;
					point.y = par_corner[y_ind].y;

					weight = face_is -> weight;
					if(y_ind == x_ind)weight = sqrt(1.0 - weight * weight);

					write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, 0.0);

					if(x_edge_is -> side != ref_position[x_ind]){

/* elliptical arc with both points on y-edge is processed */

/* CAUTION: although vertex ver_id is duplicated (no matter whether virtual or not), virtual curve bg_curve_id_minus is not duplicated 
	 because ver_id is not bounding any fixed (physical) curve */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id_minus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id_minus, tile_id);
												
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
												
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{

/* two elliptical arcs sharing point on x-edge are processed */

/* process elliptical arc closer to tile corner at first */

/* CAUTION: virtual curve bg_curve_id_plus must be duplicated because vertex ver_id is duplicated (no matter whether virtual or not)
	 and is bounding a fixed (physical) curve */

/* CAUTION: ids of created outer and inner cur_id differ by 2 * 10 !!! */

						if(interface == YES){  /* inner */
							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_plus = ++curve_id * 10, ver_id - 1, bg_vertex_id);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
										
							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2 - 1, bg_curve_id_plus, tile_id);
								
							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id3, ver_id - 1, bg_curve_id_plus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id - 1, ver_id3, bg_curve_id_plus, tile_id);
						}

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_plus = ++curve_id * 10, ver_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);
										
						if(y_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
										
						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id3, ver_id, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id, ver_id3, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);

/* then process elliptical arc further from tile corner */
									
						if(face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
							fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
							error_message(GENERAL_ERROR, "Unimplemented situation");
#endif
#ifdef DEBUG
							if(face_is -> cross_halving == Y)error_message(GENERAL_ERROR, "Unexpected situation");
#endif

/* note: patch is topologically the same no matter what halving is applied;
	 but because of its duplication in case of interface, patch is formed separately for each (implemented) halving */

							if(face_is -> cross_halving == X){                 /* halving with respect to x-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
								pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
													
								size = fabs(pnt1 -> x - pnt2 -> x);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
													
								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

/* caution: virtual vertices are not duplicated */

								if(x_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id3 = bg_vertex_id;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id4 = bg_vertex_id;
								}
													
								point3.y -= 4.0 * size;
								point4.y -= 4.0 * size;
								point5.y += 4.0 * size;
								point6.y += 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

								if(interface == YES){  /* inner */
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
													
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);

									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);

									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
										
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id - 1, bg_curve_id, tile_id);
													
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);

									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, cur_id - 2 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
								}

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
													
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
													
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, tile_id);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 2 * 10, tile_id, inclusion -> ifmat);
								}
							}		
#ifdef MIN_MAX_HALVING
							else if(face_is -> cross_halving == MIN){
							}
							else if(face_is -> cross_halving == MAX){
							}
#endif
						}
						else{

/* CAUTION: virtual curve bg_curve_id_minus must be duplicated because vertex ver_id is duplicated (no matter whether virtual or not)
	 and is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id - 1, bg_vertex_id);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);

								if(x_ind == y_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id - 1, ver_id3, bg_curve_id_minus, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id - 1, bg_curve_id_minus, tile_id);

								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, cur_id - 2 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id, bg_vertex_id);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id, ver_id3, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id3, ver_id, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, cur_id, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 2) * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 2 * 10, tile_id, inclusion -> ifmat);
							}
						}
					}
				}
			}
			else{   /* BND */
				is_point = &(y_edge_is -> point[1 - y_ind]);
				ver_id = is_point -> id;            /* outer vertex if interface == YES */

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

				if(ver_id == 0){
					point.x = par_corner[x_ind].x;
					point.y = is_point -> data.coord + shift_y; 

					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point.x, point.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);

					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point -> id = ver_id;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point -> data);
					output_y_edge_vertex = YES;
				}

/* I cannot ensure that ids of outer and inner corner vertices differ by 1 because corner was created at the beginning;
	 duplication of corner is done by fixing a virtual vertex to the corner and by fixing a physical vertex to the virtual vertex;
	 this implies that I do rely on virtual separation;
	 note that this concept introduces an overhead (virtual vertex is redundant) but this overhead cannot grow
	 because only one inclusion can be passing through a particular corner in a particular tile */

				if(interface == YES){   /* inner */
					if(cor_id == 0){
						write_record("vertex %ld fixed vertex %ld virtual\n", cor_id_aux = ++vertex_id, corner_id);
						write_record("vertex %ld fixed vertex %ld tile %ld\n", cor_id = ++vertex_id, cor_id_aux, tile_id);
					}
				}
									
				if(y_ind == 0){
					ver_id1 = corner_id;            /* outer vertex if interface == YES */
					ver_id2 = ver_id;               /* outer vertex if interface == YES */
				}
				else{
					ver_id1 = ver_id;               /* outer vertex if interface == YES */
					ver_id2 = corner_id;            /* outer vertex if interface == YES */
				}
				
				y_curve_id = get_x_side_curve_id(x_ind);
				if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

					if(y_ind == 0)
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, cor_id, ver_id2 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
					else
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1 - 1, cor_id, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
					y_curve_id += 4;
					write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
				}
				else{
					write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_y, side_spec_y);
				}
				
				y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
				output_y_edge_curve = YES;

				sign_y = (x_ind == 0) ? -1 : 1;

				if(halving_y == YES){
#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to y\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
					pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
										
					size = fabs(pnt1 -> x - pnt2 -> x);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((x_ind == 0 && center.x > par_corner[0].x) || (x_ind == 1 && center.x < par_corner[1].x)){          /* CAUTION */
#else
						if(center.x * sign[x_ind] > par_corner[x_ind].x * sign[x_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to y-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
/* caution: virtual vertices are not duplicated */

							if(x_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}												

							point3.y -= 4.0 * size;
							point4.y -= 4.0 * size;
							point5.y += 4.0 * size;
							point6.y += 4.0 * size;
				
/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to y-edge */

							point1.x = point2.x = center.x;
													
							point1.y = center.y - 2.0 * size;
							point2.y = center.y + 2.0 * size;
												
							if(x_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
												
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
												
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
												
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
							if(interface == YES){  /* inner */
								if(y_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								}
												
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
											
							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					weight = face_is -> weight;
					if(y_ind == x_ind)weight = sqrt(1.0 - weight * weight);

					//pnt1 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ on x-edge */
					pnt2 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					add_vec(point2, *pnt2, shift_vec);

					point.x = point2.x;
					point.y = par_corner[y_ind].y;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

					write_record("vertex %ld xyz "FMT3" virtual\n", bg_vertex_id = ++vertex_id, point2.x, point2.y, 0.0);

					if(interface == YES){  /* inner */
						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, cor_id, bg_vertex_id);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(y_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							
						if(x_ind == 0){
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, cor_id, ver_id4, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, cor_id, bg_curve_id, tile_id);
						}
						else{
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, cor_id, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, cor_id, ver_id3, bg_curve_id, tile_id);
						}											
							
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
					}

					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, corner_id, bg_vertex_id);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

					if(y_ind == 0)
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
					else
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
				
					if(x_ind == 0){
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id1, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id2, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					}
					else{
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id1, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id2, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					}

					if(inclusion -> mat == 0 || interface == YES)
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
					else
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

					if(interface == YES){
						write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 2) * 10, tile_id, inclusion -> ifmat);
					}
				}
			}
			tile_patch_array[tile_patches++] = patch_id;
		}
		else{
			if(inclusion -> corner == OUT){
				if((x_edge_is -> side & ref_position[x_ind]) != 0){
					is_point = &(x_edge_is -> point[0]);
					ver_id = is_point -> id;
										
					if(ver_id == 0){
						pnt = &(face_is -> extreme_pnt[2 * Y + 1 - y_ind]);          /* y- or y+ on x-edge */
						add_vec(point, *pnt, shift_vec);

						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point.x, point.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id = ++vertex_id, point.x, point.y, 0.0, tile_id);
											
						x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point -> id = ver_id;
						x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point -> data);
						output_x_edge_vertex = YES;
						x_edge_curves_2[y_ind]++;
					}
				}
			}
		}
	}
	else if(x_count == 2 && y_count == 2){   /* crossing both x-edge and y-edge */
		if((x_edge_is -> side & ref_position[x_ind]) != 0 || (y_edge_is -> side & ref_position[y_ind]) != 0){
			halving_x = halving_y = NO;
			if(inclusion -> corner == OUT){
				if((x_edge_is -> side & ref_position[x_ind]) != 0){
					is_point1 = &(x_edge_is -> point[0]);
					ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
										
					is_point2 = &(x_edge_is -> point[1]);
					ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */
										
					if(ver_id1 == 0 || ver_id2 == 0){
#ifdef DEBUG
						if(ver_id1 != 0 || ver_id2 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
						point1.y = point2.y = par_corner[y_ind].y;
											
						point1.x = is_point1 -> data.coord + shift_x; 
						point2.x = is_point2 -> data.coord + shift_x; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);

						x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
						x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
						x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point2 -> id = ver_id2;
						x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point2 -> data);
						output_x_edge_vertex = YES;
						x_edge_curves_2[y_ind]++;
					}
										
					x_curve_id = get_y_side_curve_id(y_ind);
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, ver_id2 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
						x_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id2, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id2, tile_id, output_spec, mirror_spec_x, side_spec_x);
					}
					
					x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
					output_x_edge_curve = YES;
					
					if(face_is -> edge_half[X] == YES){

/* halving is relevant only if processing part of ellipse not intersecting y-edge */

						if((y_edge_is -> side & ref_position[y_ind]) == 0)halving_x = YES;
					}
				}
				if((y_edge_is -> side & ref_position[y_ind]) != 0){
					is_point3 = &(y_edge_is -> point[0]);
					ver_id3 = is_point3 -> id;            /* outer vertex if interface == YES */
										
					is_point4 = &(y_edge_is -> point[1]);
					ver_id4 = is_point4 -> id;            /* outer vertex if interface == YES */
										
					if(ver_id3 == 0 || ver_id4 == 0){
#ifdef DEBUG
						if(ver_id3 != 0 || ver_id4 != 0)error_message(GENERAL_ERROR, "Unexpected situation");
#endif
						point3.x = point4.x = par_corner[x_ind].x;

						point3.y = is_point3 -> data.coord + shift_y; 
						point4.y = is_point4 -> data.coord + shift_y; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
											
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point3.x, point3.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point3.x, point3.y, 0.0, tile_id);
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point4.x, point4.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point4.x, point4.y, 0.0, tile_id);

						y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point3 -> id = ver_id3;
						y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point3 -> data);
						y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point4 -> id = ver_id4;
						y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point4 -> data);
						output_y_edge_vertex = YES;
						y_edge_curves_2[x_ind]++;
					}
										
					y_curve_id = get_x_side_curve_id(x_ind);
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id3 - 1, ver_id4 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
						y_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id3, ver_id4, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id3, ver_id4, tile_id, output_spec, mirror_spec_y, side_spec_y);
					}
					
					y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
					output_y_edge_curve = YES;
					
					if(face_is -> edge_half[Y] == YES){

/* halving is relevant only if processing part of ellipse not intersecting x-edge */

						if((x_edge_is -> side & ref_position[x_ind]) == 0)halving_y = YES;
					}
				}
				
				sign_x = (y_ind == 1) ? -1 : 1;
				sign_y = (x_ind == 0) ? -1 : 1;
				
				if(halving_x == YES){

/* half-ellipse arc with both points "on" x-edge is processed */

#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to x\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
										
					size = fabs(pnt1 -> y - pnt2 -> y);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((y_ind == 0 && center.y > par_corner[0].y) || (y_ind == 1 && center.y < par_corner[1].y)){          /* CAUTION */
#else
						if(center.y * sign[y_ind] > par_corner[y_ind].y * sign[y_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to x-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
/* caution: virtual vertices are not duplicated */

							if(y_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}
											
							point3.x += 4.0 * size;
							point4.x += 4.0 * size;
							point5.x -= 4.0 * size;
							point6.x -= 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id6, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}
								
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to x-edge */

							point1.y = point2.y = center.y;
											
							point1.x = center.x - 2.0 * size;
							point2.x = center.x + 2.0 * size;
											
							if(y_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}

							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
											
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
												
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);

							if(interface == YES){  /* inner */
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
											
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
											
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else if(halving_y == YES){

/* half-ellipse arc with both points "on" y-edge is processed */

#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to y\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
					pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
										
					size = fabs(pnt1 -> x - pnt2 -> x);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((x_ind == 0 && center.x > par_corner[0].x) || (x_ind == 1 && center.x < par_corner[1].x)){          /* CAUTION */
#else
						if(center.x * sign[x_ind] > par_corner[x_ind].x * sign[x_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to y-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
				
/* caution: virtual vertices are not duplicated */
	
							if(x_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0);
							}

							point3.y -= 4.0 * size;
							point4.y -= 4.0 * size;
							point5.y += 4.0 * size;
							point6.y += 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

							if(interface == YES){  /* inner */
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id1, ver_id2 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id1 - 1, ver_id2);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id2 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id1 - 1, ver_id5, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id2 - 1, ver_id1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id2, ver_id1 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2 - 1, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id1 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id1, ver_id2);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id2, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id1, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id2, ver_id1);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id2, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id1, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to y-edge */

							point1.x = point2.x = center.x;
													
							point1.y = center.y - 2.0 * size;
							point2.y = center.y + 2.0 * size;
												
							if(x_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
												
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
												
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0);
												
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id1, ver_id2);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
							if(interface == YES){  /* inner */
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3 - 1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4 - 1, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id3, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id4, bg_curve_id, tile_id);
												
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					pnt1 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ */
											
					add_vec(point1, *pnt1, shift_vec);
					add_vec(point2, *pnt2, shift_vec);
											
					point.x = point1.x;
					point.y = point2.y;
											
					weight = face_is -> weight;
					if(x_ind == y_ind)weight = sqrt(1.0 - weight * weight);
											
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id9 = ++vertex_id, point1.x, point1.y, 0.0);
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id10 = ++vertex_id, point2.x, point2.y, 0.0);

					if(y_edge_is -> side != ref_position[y_ind]){

/* elliptical arc with both points on x-edge is processed */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id9, ver_id10);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);
								
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id_minus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id_minus, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);

						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);

						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else if(x_edge_is -> side != ref_position[x_ind]){           

/* elliptical arc with both points on y-edge is processed */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id9, ver_id10);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3 - 1, bg_curve_id_minus, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4 - 1, bg_curve_id_minus, tile_id);
								
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id7, ver_id8, bg_curve_id_minus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id8, ver_id7, bg_curve_id_minus, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3, bg_curve_id_minus, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4, bg_curve_id_minus, tile_id);

						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id7, ver_id8, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id8, ver_id7, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
												
						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{

/* two elliptical arcs joining points on x-edge and y-edge are processed */

/* process elliptical arc closer to tile corner at first */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_plus = ++curve_id * 10, ver_id9, ver_id10);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, weight);

/* IMPORTANT: ensure that ids of created outer and inner cur_id differ by 1 * 10 !!! */
											
						if(interface == YES){  /* inner */
							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2 - 1, bg_curve_id_plus, tile_id);
								
							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3 - 1, bg_curve_id_plus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id4 - 1, bg_curve_id_plus, tile_id);
								
							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id7, ver_id5, bg_curve_id_plus, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id = ++curve_id * 10, ver_id5, ver_id7, bg_curve_id_plus, tile_id);
						}

						if(x_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id_plus, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id_plus, tile_id);
											
						if(y_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id3, bg_curve_id_plus, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id7 = ++vertex_id, ver_id4, bg_curve_id_plus, tile_id);
											
						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id7, ver_id5, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id = ++curve_id * 10, ver_id5, ver_id7, bg_curve_id_plus, CURVE_RATE, inclusion -> bmsz, tile_id);

/* then process elliptical arc further from tile corner */

						if(face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
							fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
							error_message(GENERAL_ERROR, "Unimplemented situation");
#endif

/* then process elliptical arc further from tile corner */

/* note: patch is topologically the same no matter what halving is applied;
	 but because of its duplication in case of interface, patch is formed separately for each (implemented) halving */

							if(face_is -> cross_halving == Y){                 /* halving with respect to y-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
								pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
									
								size = fabs(pnt1 -> y - pnt2 -> y);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

								if(y_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id5 = ver_id10;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id6 = ver_id10;
								}
												
								point3.x += 4.0 * size;
								point4.x += 4.0 * size;
								point5.x -= 4.0 * size;
								point6.x -= 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

								if(interface == YES){  /* inner */
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5 - 1, ver_id6);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									else{
										if(y_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4 - 1, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3 - 1, bg_curve_id, tile_id);
									}
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5 - 1, ver_id, bg_curve_id, tile_id);
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6 - 1, ver_id5);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
										
									if(x_ind == 0){
										if(y_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4 - 1, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3 - 1, bg_curve_id, tile_id);
									}
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6 - 1, ver_id, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5 - 1, bg_curve_id, tile_id);
						
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, cur_id - 1 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
								}
										
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else{
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
								}
												
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(x_ind == 0){
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
								}
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);

								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 1 * 10, tile_id, inclusion -> ifmat);
								}
							}
							else if(face_is -> cross_halving == X){            /* halving with respect to x-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
								pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
												
								size = fabs(pnt1 -> x - pnt2 -> x);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
												
								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

								if(x_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id5 = ver_id9;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id6 = ver_id9;
								}
												
								point3.y -= 4.0 * size;
								point4.y -= 4.0 * size;
								point5.y += 4.0 * size;
								point6.y += 4.0 * size;

/* CAUTION: virtual curve bg_curve_id must be duplicated because its physical vertex is bounding a fixed (physical) curve */

								if(interface == YES){  /* inner */
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5 - 1, ver_id6);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
									if(y_ind == 0){
										if(x_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									}
									else // HUHU
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3 - 1, bg_curve_id, tile_id);
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5 - 1, ver_id, bg_curve_id, tile_id);
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6 - 1, ver_id5);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
										
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4 - 1, bg_curve_id, tile_id);
									else{
										if(x_ind == 0)
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										else
											write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									}
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6 - 1, ver_id, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5 - 1, bg_curve_id, tile_id);
										
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, cur_id - 1 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
								}	

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
								if(y_ind == 0){
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								}
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id3, bg_curve_id, tile_id);
												
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id4, bg_curve_id, tile_id);
								else{
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								}

								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, tile_id);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 1 * 10, tile_id, inclusion -> ifmat);
								}
							}		
#ifdef MIN_MAX_HALVING
							else if(face_is -> cross_halving == MIN){
							}
							else if(face_is -> cross_halving == MAX){
							}
#endif
						}
						else{
							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id_minus = ++curve_id * 10, ver_id9, ver_id10);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
										
/* IMPORTANT: fixed vertices are HERE not numbered increasingly along edge but increasingly from tile corner !!!
	 this will simplify curve forming */

							if(interface == YES){  /* inner */
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1 - 1, bg_curve_id_minus, tile_id);
									
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4 - 1, bg_curve_id_minus, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id3 - 1, bg_curve_id_minus, tile_id);
									
								if(x_ind == y_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id8, bg_curve_id_minus, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id8, ver_id6, bg_curve_id_minus, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, curve_id * 10, cur_id - 1 * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id_minus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id_minus, tile_id);

							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id4, bg_curve_id_minus, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id8 = ++vertex_id, ver_id3, bg_curve_id_minus, tile_id);
								
							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id6, ver_id8, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id8, ver_id6, bg_curve_id_minus, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, cur_id, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, cur_id, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id, cur_id - 1 * 10, tile_id, inclusion -> ifmat);
							}
						}
					}
				}
			}
			else if(inclusion -> corner == BND){
				ver_id = corner_id;            /* outer vertex if interface == YES */

/* I cannot ensure that ids of outer and inner corner vertices differ by 1 because corner was created at the beginning;
	 duplication of corner is done by fixing a virtual vertex to the corner and by fixing a physical vertex to the virtual vertex;
	 this implies that I do rely on virtual separation;
	 note that this concept introduces an overhead (virtual vertex is redundant) but this overhead cannot grow
	 because only one inclusion can be passing through a particular corner in a particular tile */

				if(interface == YES){   /* inner */
					if(cor_id == 0){
						write_record("vertex %ld fixed vertex %ld virtual\n", cor_id_aux = ++vertex_id, corner_id);
						write_record("vertex %ld fixed vertex %ld tile %ld\n", cor_id = ++vertex_id, cor_id_aux, tile_id);
					}
				}
									
				if((x_edge_is -> side & ref_position[x_ind]) != 0){
					is_point1 = &(x_edge_is -> point[1 - x_ind]);
					ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
										
					if(ver_id1 == 0){
						point1.x = is_point1 -> data.coord + shift_x; 
						point1.y = par_corner[y_ind].y;

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
											
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);

						x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
						x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
						output_x_edge_vertex = YES;
					}
										
					x_curve_id = get_y_side_curve_id(y_ind);
					if(x_ind == 0){
						if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, cor_id, ver_id1 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
							x_curve_id += 4;
							write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id, ver_id1, tile_id);  /* outer */
						}
						else{
							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id, ver_id1, tile_id, output_spec, mirror_spec_x, side_spec_x);
						}
					}
					else{
						if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, cor_id, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
							x_curve_id += 4;
							write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id, tile_id);  /* outer */
						}
						else{
							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id, tile_id, output_spec, mirror_spec_x, side_spec_x);
						}
					}
					
					x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
					output_x_edge_curve = YES;

					if(face_is -> edge_half[X] == YES){

/* halving is relevant only if processing part of ellipse not intersecting y-edge */

						if((y_edge_is -> side & ref_position[y_ind]) == 0)halving_x = YES;
					}
				}
				if((y_edge_is -> side & ref_position[y_ind]) != 0){
					is_point2 = &(y_edge_is -> point[1 - y_ind]);
					ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */
										
					if(ver_id2 == 0){
						point2.x = par_corner[x_ind].x;
						point2.y = is_point2 -> data.coord + shift_y; 

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
												
						if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
						write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						
						y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
						y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
						output_y_edge_vertex = YES;
					}
					
					y_curve_id = get_x_side_curve_id(x_ind);
					if(y_ind == 0){
						if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, cor_id, ver_id2 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
							y_curve_id += 4;
							write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id, ver_id2, tile_id);  /* outer */
						}
						else{
							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id, ver_id2, tile_id, output_spec, mirror_spec_y, side_spec_y);
						}
					}
					else{
						if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id2 - 1, cor_id, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
							y_curve_id += 4;
							write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id2, ver_id, tile_id);  /* outer */
						}
						else{
							write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id2, ver_id, tile_id, output_spec, mirror_spec_y, side_spec_y);
						}
					}
					
					y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
					output_y_edge_curve = YES;

					if(face_is -> edge_half[Y] == YES){

/* halving is relevant only if processing part of ellipse not intersecting x-edge */

						if((x_edge_is -> side & ref_position[x_ind]) == 0)halving_y = YES;
					}
				}
				
				sign_x = (y_ind == 1) ? -1 : 1;
				sign_y = (x_ind == 0) ? -1 : 1;
				
				if(halving_x == YES){

/* half-ellipse arc with both points "on" x-edge is processed */

#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to x\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
										
					size = fabs(pnt1 -> y - pnt2 -> y);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((y_ind == 0 && center.y > par_corner[0].y) || (y_ind == 1 && center.y < par_corner[1].y)){          /* CAUTION */
#else
						if(center.y * sign[y_ind] > par_corner[y_ind].y * sign[y_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to x-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

							if(y_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}

							point3.x += 4.0 * size;
							point4.x += 4.0 * size;
							point5.x -= 4.0 * size;
							point6.x -= 4.0 * size;

							if(interface == YES){  /* inner */
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id6, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
									
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
											
							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to x-edge */

							point1.y = point2.y = center.y;

							point1.x = center.x - 2.0 * size;
							point2.x = center.x + 2.0 * size;
												
							if(y_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
								
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
								
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
								
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
								
							if(interface == YES){  /* inner */
								if(x_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								}
									
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							if(x_ind == 0){
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							}
							else{
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							}

							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								
							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else if(halving_y == YES){

/* half-ellipse arc with both points "on" y-edge is processed */

#ifdef DEBUG_PRINT
					fprintf(stderr, "halving with respect to y\n\n");
#endif
					pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
					pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
										
					size = fabs(pnt1 -> x - pnt2 -> x);
					size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
#if 0
					if((x_ind == 0 && center.x > par_corner[0].x) || (x_ind == 1 && center.x < par_corner[1].x)){          /* CAUTION */
#else
						if(center.x * sign[x_ind] > par_corner[x_ind].x * sign[x_ind]){   /* CAUTION: floating point number based decision */
#endif

/* halving "perpendicularly" to y-edge */

							add_vec(point1, *pnt1, shift_vec);
							add_vec(point2, *pnt2, shift_vec);
							copy_vec(point3, point1);
							copy_vec(point4, point2);
							copy_vec(point5, point1);
							copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

							if(x_ind == 0){
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
							}
							else{
								if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
								write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
								write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
							}												

							point3.y -= 4.0 * size;
							point4.y -= 4.0 * size;
							point5.y += 4.0 * size;
							point6.y += 4.0 * size;

							if(interface == YES){  /* inner */
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
									
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4 - 1, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3 - 1, ver_id5, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
								else
									write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3 - 1, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id3, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);

							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id6, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
							}
						}
						else{

/* halving parallel to y-edge */

							point1.x = point2.x = center.x;
													
							point1.y = center.y - 2.0 * size;
							point2.y = center.y + 2.0 * size;
												
							if(x_ind == 0){
								sub_vec(vec, *pnt2, *pnt1);
							}
							else{
								sub_vec(vec, *pnt1, *pnt2);
							}
								
							add_vec(point3, point1, vec);
							add_vec(point4, point2, vec);
								
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
								
							write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
								
							if(interface == YES){  /* inner */
								if(y_ind == 0){
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, cor_id, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								}
								else{
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, cor_id, bg_curve_id, tile_id);
								}
									
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							if(y_ind == 0){
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							}
							else{
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id, bg_curve_id, tile_id);
							}
												
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
#if 0 /* only to fix indentation */
						}
#else
					}
#endif
				}
				else{
					pnt1 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ */
											
					add_vec(point1, *pnt1, shift_vec);
					add_vec(point2, *pnt2, shift_vec);
											
					point.x = point1.x;
					point.y = point2.y;
											
					weight = face_is -> weight;
					if(x_ind == y_ind)weight = sqrt(1.0 - weight * weight);
											
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id9 = ++vertex_id, point1.x, point1.y, 0.0);
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id10 = ++vertex_id, point2.x, point2.y, 0.0);

					if(y_edge_is -> side != ref_position[y_ind]){

/* elliptical arc with both points on x-edge is processed */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id9, ver_id10);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, cor_id, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);

							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id1, bg_curve_id, tile_id);

						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else if(x_edge_is -> side != ref_position[x_ind]){           

/* elliptical arc with both points on y-edge is processed */

						write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id9, ver_id10);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

						if(interface == YES){  /* inner */
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, cor_id, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
							
							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g density 2 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld tile %ld hole\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld size %g tile %ld property %d\n", ++patch_id, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
						}
					}
					else{

/* elliptical arc joining points on x-edge and y-edge is processed */

						if(face_is -> cross_half == YES){
#ifdef DEBUG_PRINT												
							fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
							error_message(GENERAL_ERROR, "Unimplemented situation");
#endif

/* then process elliptical arc further from tile corner */

/* note: patch is topologically the same no matter what halving is applied;
	 but because of its duplication in case of interface, patch is formed separately for each (implemented) halving */

							if(face_is -> cross_halving == Y){                 /* halving with respect to y-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
								pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
									
								size = fabs(pnt1 -> y - pnt2 -> y);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */

								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

								if(y_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id5 = ver_id10;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id6 = ver_id10;
								}
												
								point3.x += 4.0 * size;
								point4.x += 4.0 * size;
								point5.x -= 4.0 * size;
								point6.x -= 4.0 * size;

								if(interface == YES){  /* inner */
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5 - 1, ver_id6);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5 - 1, ver_id, bg_curve_id, tile_id);
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6 - 1, ver_id5);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
										
									if(x_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
										
									if(y_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6 - 1, ver_id, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5 - 1, bg_curve_id, tile_id);
										
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
								}

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(x_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
												
								if(y_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
								}
							}
							else if(face_is -> cross_halving == X){            /* halving with respect to x-extreme point */
								pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
								pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
												
								size = fabs(pnt1 -> x - pnt2 -> x);
								size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */
												
								add_vec(point1, *pnt1, shift_vec);
								add_vec(point2, *pnt2, shift_vec);
								copy_vec(point3, point1);
								copy_vec(point4, point2);
								copy_vec(point5, point1);
								copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

								if(x_ind == 0){
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0);
									ver_id5 = ver_id9;
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
								}
								else{
									if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
									write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id5 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
									//write_record("vertex %ld xyz "FMT3" virtual\n", ver_id6 = ++vertex_id, point2.x, point2.y, 0.0);
									ver_id6 = ver_id9;
								}
												
								point3.y -= 4.0 * size;
								point4.y -= 4.0 * size;
								point5.y += 4.0 * size;
								point6.y += 4.0 * size;

								if(interface == YES){  /* inner */
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6 - 1);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5 - 1, ver_id6);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
										
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6 - 1, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5 - 1, ver_id, bg_curve_id, tile_id);
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6 - 1, ver_id5);
									else
										write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5 - 1);
									write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
									write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
										
									if(y_ind == 0)
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
									else
										write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
										
									if(x_ind == 0)
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6 - 1, ver_id, bg_curve_id, tile_id);
									else
										write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5 - 1, bg_curve_id, tile_id);
										
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
								}

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id5, ver_id6);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
												
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id5, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id6, ver_id5);
								write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
								write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

								if(y_ind == 0)
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								else
									write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
												
								if(x_ind == 0)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id6, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

								if(inclusion -> mat == 0 || interface == YES)
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
								else
									write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

								if(interface == YES){
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
									write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
								}
							}		
#ifdef MIN_MAX_HALVING
							else if(face_is -> cross_halving == MIN){
							}
							else if(face_is -> cross_halving == MAX){
							}
#endif
						}
						else{
							write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id9, ver_id10);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);

							if(interface == YES){  /* inner */
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);

								if(x_ind == y_ind)
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, tile_id);
								else
									write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, tile_id);
									
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
							}

							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id3 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id4 = ++vertex_id, ver_id2, bg_curve_id, tile_id);

							if(x_ind == y_ind)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id3, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id4, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

							if(inclusion -> mat == 0 || interface == YES)
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, tile_id);
							else
								write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

							if(interface == YES){
								write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
							}
						}
					}
				}
			}
			else{  /* IN */
				ver_id = corner_id;            /* outer vertex if interface == YES */

/* I cannot ensure that ids of outer and inner corner vertices differ by 1 because corner was created at the beginning;
	 duplication of corner is done by fixing a virtual vertex to the corner and by fixing a physical vertex to the virtual vertex;
	 this implies that I do rely on virtual separation;
	 note that this concept introduces an overhead (virtual vertex is redundant) but this overhead cannot grow
	 because only one inclusion can be passing through a particular corner in a particular tile */

				if(interface == YES){   /* inner */
					if(cor_id == 0){
						write_record("vertex %ld fixed vertex %ld virtual\n", cor_id_aux = ++vertex_id, corner_id);
						write_record("vertex %ld fixed vertex %ld tile %ld\n", cor_id = ++vertex_id, cor_id_aux, tile_id);
					}
				}
									
/* CAUTION: this is the case when intersection with all three edges occurs and each edge_is has side equal to BOTH;
	 this implies that is_points have coord already subjected to periodic repetition and only relevant tile shift has to be applied */
										
				is_point1 = &(x_edge_is -> point[1 - x_ind]);
				ver_id1 = is_point1 -> id;            /* outer vertex if interface == YES */
									
				if(ver_id1 == 0){
					if(x_ind == ind_x)               /* <==> per_x == 0 */
						point1.x = is_point1 -> data.coord + shift_x;               /* shift_x does not contain periodic repetition */
					else                             /* <==> per_x == 1 */
						point1.x = is_point1 -> data.coord + shift_x - sign[ind_x] * tile.size.x;     /* remove periodic repetition */
					point1.y = par_corner[y_ind].y;
	
/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
									
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id1 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
					
					x_edge_vertex_id_array[y_ind][x_edge_vertices[y_ind]] = is_point1 -> id = ver_id1;
					x_edge_vertex_data_array[y_ind][x_edge_vertices[y_ind]++] = &(is_point1 -> data);
					output_x_edge_vertex = YES;
				}
				
				x_curve_id = get_y_side_curve_id(y_ind);
				if(x_ind == 0){
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, cor_id, ver_id1 - 1, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
						x_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id, ver_id1, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id, ver_id1, tile_id, output_spec, mirror_spec_x, side_spec_x);
					}
				}
				else{
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner x_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1 - 1, cor_id, tile_id, output_spec, mirror_spec_x, side_spec_x);  /* inner */
						x_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", x_curve_id, ver_id1, ver_id, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", x_curve_id, ver_id1, ver_id, tile_id, output_spec, mirror_spec_x, side_spec_x);
					}
				}
				
				x_edge_curve_1_array[y_ind][x_edge_curves_1[y_ind]++] = x_curve_id;
				output_x_edge_curve = YES;
									
				is_point2 = &(y_edge_is -> point[1 - y_ind]);
				ver_id2 = is_point2 -> id;            /* outer vertex if interface == YES */
									
				if(ver_id2 == 0){
					point2.x = par_corner[x_ind].x;
					if(y_ind == ind_y)               /* <==> per_y == 0 */
						point2.y = is_point2 -> data.coord + shift_y;               /* shift_y does not contain periodic repetition */
					else                             /* <==> per_y == 1 */
						point2.y = is_point2 -> data.coord + shift_y - sign[ind_y] * tile.size.y;     /* remove periodic repetition */
		
/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
		
					if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
					write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id2 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
					
					y_edge_vertex_id_array[x_ind][y_edge_vertices[x_ind]] = is_point2 -> id = ver_id2;
					y_edge_vertex_data_array[x_ind][y_edge_vertices[x_ind]++] = &(is_point2 -> data);
					output_y_edge_vertex = YES;
				}
									
				y_curve_id = get_x_side_curve_id(x_ind);
				if(y_ind == 0){
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, cor_id, ver_id2 - 1, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
						y_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id, ver_id2, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id, ver_id2, tile_id, output_spec, mirror_spec_y, side_spec_y);
					}
				}
				else{
					if(interface == YES){

/* IMPORTANT: ensure that ids of created outer and inner y_curve differ by 4 !!! */

						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id2 - 1, cor_id, tile_id, output_spec, mirror_spec_y, side_spec_y);  /* inner */
						y_curve_id += 4;
						write_record("curve %ld vertex %ld %ld tile %ld output no\n", y_curve_id, ver_id2, ver_id, tile_id);  /* outer */
					}
					else{
						write_record("curve %ld vertex %ld %ld tile %ld %s %s %s\n", y_curve_id, ver_id2, ver_id, tile_id, output_spec, mirror_spec_y, side_spec_y);
					}
				}
				
				y_edge_curve_1_array[x_ind][y_edge_curves_1[x_ind]++] = y_curve_id;
				output_y_edge_curve = YES;
									
				sign_y = (x_ind == 0) ? -1 : 1;
				sign_x = (y_ind == 1) ? -1 : 1;

				if((cross_halving = face_is -> cross_half) == YES){

/* cross halving is relevant only in particular quadrant;
	 note that I cannot use edge_is -> side as its value is BOTH for both edges;
	 therefore I have to check for coordinates of ellipse center with respect to tile corner;
	 although flouting point number based desicion is made, it should not be sensitive to round-off error 
	 because ellipse center is safely inside particular quadrant */

#if 0
					if((x_ind == 0 && center.x <= par_corner[0].x) || (x_ind == 1 && center.x >= par_corner[1].x))cross_halving = NO;
					if((y_ind == 0 && center.y <= par_corner[0].y) || (y_ind == 1 && center.y >= par_corner[1].y))cross_halving = NO;
#else
					if(center.x * sign[x_ind] <= par_corner[x_ind].x * sign[x_ind])cross_halving = NO;
					if(center.y * sign[y_ind] <= par_corner[y_ind].y * sign[y_ind])cross_halving = NO;
#endif
				}

				if(cross_halving == YES){
#ifdef DEBUG_PRINT												
					fprintf(stderr, "cross halving\n\n");
#endif
#ifdef MIN_MAX_HALVING
					error_message(GENERAL_ERROR, "Unimplemented situation");
#endif

/* note: patch is topologically the same no matter what halving is applied;
	 but because of its duplication in case of interface, patch is formed separately for each (implemented) halving */

					if(face_is -> cross_halving == Y){                 /* halving with respect to y-extreme point */
						pnt1 = &(face_is -> extreme_pnt[2 * Y + 0]);     /* y- */
						pnt2 = &(face_is -> extreme_pnt[2 * Y + 1]);     /* y+ */
								
						size = fabs(pnt1 -> y - pnt2 -> y);
						size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */

						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
						copy_vec(point3, point1);
						copy_vec(point4, point2);
						copy_vec(point5, point1);
						copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */

						if(y_ind == 0){
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						}
						else{
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
						}
												
						point3.x += 4.0 * size;
						point4.x += 4.0 * size;
						point5.x -= 4.0 * size;
						point6.x -= 4.0 * size;

						if(interface == YES){  /* inner */
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
								
							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id4 - 1, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3 - 1, ver_id, bg_curve_id, tile_id);
								
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
								
							if(x_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								
							if(y_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id3 - 1, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
			
						if(x_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
							
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

						if(x_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							
						if(y_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
						}
					}
					else if(face_is -> cross_halving == X){            /* halving with respect to x-extreme point */
						pnt1 = &(face_is -> extreme_pnt[2 * X + 0]);     /* x- */
						pnt2 = &(face_is -> extreme_pnt[2 * X + 1]);     /* x+ */
											
						size = fabs(pnt1 -> x - pnt2 -> x);
						size = ellipse -> max * ellipse -> min / size;   /* constant area defined by associated diameters */

						add_vec(point1, *pnt1, shift_vec);
						add_vec(point2, *pnt2, shift_vec);
						copy_vec(point3, point1);
						copy_vec(point4, point2);
						copy_vec(point5, point1);
						copy_vec(point6, point2);

/* IMPORTANT: ensure that ids of created outer and inner vertices differ by 1 !!! */
											
						if(x_ind == 0){
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point2.x, point2.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0, tile_id);
						}
						else{
							if(interface == YES)write_record("vertex %ld xyz "FMT3" tile %ld\n", ++vertex_id, point1.x, point1.y, 0.0, tile_id);  /* inner */
							write_record("vertex %ld xyz "FMT3" tile %ld\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0, tile_id);
							write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
						}
											
						point3.y -= 4.0 * size;
						point4.y -= 4.0 * size;
						point5.y += 4.0 * size;
						point6.y += 4.0 * size;

						if(interface == YES){  /* inner */
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4 - 1);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3 - 1, ver_id4);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
								
							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
								
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id4 - 1, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3 - 1, ver_id, bg_curve_id, tile_id);
								
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4 - 1, ver_id3);
							else
								write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3 - 1);
							write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
							write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);
								
							if(y_ind == 0)
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
							else
								write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
								
							if(x_ind == 0)
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4 - 1, ver_id, bg_curve_id, tile_id);
							else
								write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id3 - 1, bg_curve_id, tile_id);
								
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);
						}

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point3.x, point3.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point4.x, point4.y, 0.0, 1.0 / 3.0);
												
						if(y_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
								
						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id, ver_id4, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id1 = ++curve_id * 10, ver_id3, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						write_record("curve %ld vertex %ld %ld order 4 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id4, ver_id3);
						write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point6.x, point6.y, 0.0, 1.0 / 3.0);
						write_record("polygon 2 xyz "FMT3" weight "FMT1"\n", point5.x, point5.y, 0.0, 1.0 / 3.0);

						if(y_ind == 0)
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id2, bg_curve_id, tile_id);
						else
							write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id = ++vertex_id, ver_id1, bg_curve_id, tile_id);
							
						if(x_ind == 0)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id4, ver_id, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", cur_id2 = ++curve_id * 10, ver_id, ver_id3, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);

						if(inclusion -> mat == 0 || interface == YES)
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, tile_id);
						else
							write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, cur_id1, cur_id2, inclusion -> imsz, tile_id, inclusion -> mat);

						if(interface == YES){
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id1, cur_id1 - 4 * 10, tile_id, inclusion -> ifmat);
							write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, cur_id2, cur_id2 - 4 * 10, tile_id, inclusion -> ifmat);
						}
					}
#ifdef MIN_MAX_HALVING
					else if(face_is -> cross_halving == MIN){
					}
					else if(face_is -> cross_halving == MAX){
					}
#endif
				}
				else{
					pnt1 = &(face_is -> extreme_pnt[2 * X + x_ind]);        /* x- or x+ */
					pnt2 = &(face_is -> extreme_pnt[2 * Y + y_ind]);        /* y- or y+ */
									
					add_vec(point1, *pnt1, shift_vec);
					add_vec(point2, *pnt2, shift_vec);
									
					point.x = point1.x;
					point.y = point2.y;
										
					weight = face_is -> weight;
					if(x_ind == y_ind)weight = sqrt(1.0 - weight * weight);
										
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id3 = ++vertex_id, point1.x, point1.y, 0.0);
					write_record("vertex %ld xyz "FMT3" virtual\n", ver_id4 = ++vertex_id, point2.x, point2.y, 0.0);
					write_record("curve %ld vertex %ld %ld order 3 rate 0 factor 0 virtual\n", bg_curve_id = ++curve_id * 10, ver_id3, ver_id4);
					write_record("polygon 1 xyz "FMT3" weight "FMT1"\n", point.x, point.y, 0.0, -weight);
										
					if(interface == YES){  /* inner */
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1 - 1, bg_curve_id, tile_id);
						write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2 - 1, bg_curve_id, tile_id);
										
						if(x_ind == y_ind)
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, tile_id);
						else
							write_record("curve %ld vertex %ld %ld fixed curve %ld rate 0 factor 0 tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, tile_id);
							
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, (x_curve_id - 4) * sign_x, (y_curve_id - 4) * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);
					}

					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id5 = ++vertex_id, ver_id1, bg_curve_id, tile_id);
					write_record("vertex %ld fixed vertex %ld fixed curve %ld tile %ld\n", ver_id6 = ++vertex_id, ver_id2, bg_curve_id, tile_id);
										
					if(x_ind == y_ind)
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id5, ver_id6, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
					else
						write_record("curve %ld vertex %ld %ld fixed curve %ld rate %g size %g tile %ld\n", ++curve_id * 10, ver_id6, ver_id5, bg_curve_id, CURVE_RATE, inclusion -> bmsz, tile_id);
										
					if(inclusion -> mat == 0 || interface == YES)
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld tile %ld hole\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, tile_id);
					else
						write_record("patch %ld normal 0 0 1 boundary curve %ld %ld %ld size %g tile %ld property %d\n", ++patch_id, x_curve_id * sign_x, y_curve_id * sign_y, curve_id * 10, inclusion -> imsz, tile_id, inclusion -> mat);

					if(interface == YES){
						write_record("interface %ld type curve %ld %ld tile %ld property %d mirror simple copy\n", ++interface_id, curve_id * 10, (curve_id - 1) * 10, tile_id, inclusion -> ifmat);
					}
				}
			}
			tile_patch_array[tile_patches++] = patch_id;
		}
	}
	else
		error_message(GENERAL_ERROR, "Unexpected situation");

	if(output_x_edge_vertex == YES){
		if(x_edge_vertices[y_ind] > max_edge_vertices[X])error_message(GENERAL_ERROR, "Unexpected situation");
	}
	if(output_y_edge_vertex == YES){
		if(y_edge_vertices[x_ind] > max_edge_vertices[Y])error_message(GENERAL_ERROR, "Unexpected situation");
	}
	if(output_x_edge_curve == YES){
		if(x_edge_curves_1[y_ind] > max_edge_curves_1[X])error_message(GENERAL_ERROR, "Unexpected situation");
	}
	if(output_y_edge_curve == YES){
		if(y_edge_curves_1[x_ind] > max_edge_curves_1[Y])error_message(GENERAL_ERROR, "Unexpected situation");
	}
}



/* this function replicates checks in process_geometry[_without|with_interface] in above included source files */

static logic
check_inclusion_outside_tile(void)
{
	logic outside = YES;

	x_edge_is = inclusion -> edge_is[X];
	y_edge_is = inclusion -> edge_is[Y];
		
	if(ellipse -> max == 0.0){                 /* side touching */
		if(x_count == 0 && y_count == 0){        /* not touching edge */
			if((face_is -> side[X] & ref_position[x_ind]) != 0 && (face_is -> side[Y] & ref_position[y_ind]) != 0)outside = NO;
		}
		else{                                    /* touching edge */
			if(y_count == 0){                      /* touching x-edge */
				if((x_edge_is -> side & ref_position[x_ind]) != 0)
					outside = NO;                      /* touching x-edge from inside */
				else
					outside = NO;                      /* touching x-edge from outside */
			}
			else if(x_count == 0){                 /* touching y-edge */
				if((y_edge_is -> side & ref_position[y_ind]) != 0)
					outside = NO;                      /* touching y-edge from inside */
				else
					outside = NO;                      /* touching y-edge from outside */
			}
			else                                   /* touching corner */
				outside = NO;
		}
	}
	else{
		if(x_count == 0 && y_count == 0){        /* not crossing edges */
			if((face_is -> side[X] & ref_position[x_ind]) != 0 && (face_is -> side[Y] & ref_position[y_ind]) != 0)outside = NO;
		}
		else if(x_count == 1 && y_count == 0){   /* touching x-edge */
			if((x_edge_is -> side & ref_position[x_ind]) != 0){
				if((face_is -> side[Y] & ref_position[y_ind]) != 0)
					outside = NO;                      /* touching x-edge from inside */
				else
					outside = NO;                      /* touching x-edge from outside */
			}
		}
		else if(x_count == 0 && y_count == 1){   /* touching y-edge */
			if((y_edge_is -> side & ref_position[y_ind]) != 0){
				if((face_is -> side[X] & ref_position[x_ind]) != 0)
					outside = NO;                      /* touching y-edge from inside */
				else
					outside = NO;                      /* touching y-edge from outside */
			}
		}
		else if(x_count == 1 && y_count == 1){   /* touching both x-edge and y-edge */
			if((x_edge_is -> side & ref_position[x_ind]) != 0 && (y_edge_is -> side & ref_position[y_ind]) != 0)
				outside = NO;
			else{
				if((x_edge_is -> side & ref_position[x_ind]) != 0)outside = NO;
				if((y_edge_is -> side & ref_position[y_ind]) != 0)outside = NO;
			}
		}
		else if(x_count == 2 && y_count == 0){   /* crossing x-edge */
			if((x_edge_is -> side & ref_position[x_ind]) != 0)outside = NO;
		}
		else if(x_count == 0 && y_count == 2){   /* crossing y-edge */
			if((y_edge_is -> side & ref_position[y_ind]) != 0)outside = NO;
		}
		else if(x_count == 2 && y_count == 1){   /* crossing x-edge, touching y-edge */
			if((x_edge_is -> side & ref_position[x_ind]) != 0)
				outside = NO;
			else{
				if(inclusion -> corner == OUT){
					if((y_edge_is -> side & ref_position[y_ind]) != 0)outside = NO;
				}
			}
		}
		else if(x_count == 1 && y_count == 2){   /* crossing y-edge, touching x-edge */
			if((y_edge_is -> side & ref_position[y_ind]) != 0)
				outside = NO;
			else{
				if(inclusion -> corner == OUT){
					if((x_edge_is -> side & ref_position[x_ind]) != 0)outside = NO;
				}
			}
		}
		else if(x_count == 2 && y_count == 2){   /* crossing both x-edge and y-edge */
			if((x_edge_is -> side & ref_position[x_ind]) != 0 || (y_edge_is -> side & ref_position[y_ind]) != 0)outside = NO;
		}
		else
			error_message(GENERAL_ERROR, "Unexpected situation");
	}

	return(outside);
}
